@@ -117,132 +117,70 @@ static void phi_identity(element_ptr out, element_ptr in, pairing_ptr pairing)
     element_set(out, in);
 }
 
-static void point_proj_double(point_t R, element_t Rz, point_t P, element_t z)
-{
-    element_t e0, e1, e2, e3;
-    element_init(e0, z->field);
-    element_init(e1, z->field);
-    element_init(e2, z->field);
-    element_init(e3, z->field);
-    element_ptr x = P->x;
-    element_ptr y = P->y;
-    //e0 = 3x^2 + (cc->a) z^4
-    element_square(e0, x);
-    element_mul_si(e0, e0, 3);
-    element_square(e1, z);
-    element_square(e1, e1);
-    element_add(e0, e0, e1);
-
-    //z_out = 2 y z
-    element_mul(Rz, y, z);
-    element_mul_si(Rz, Rz, 2);
-
-    //e1 = 4 x y^2
-    element_square(e2, y);
-    element_mul(e1, x, e2);
-    element_mul_si(e1, e1, 4);
-
-    //x_out = e0^2 - 2 e1
-    element_mul_si(e3, e1, 2);
-    element_square(R->x, e0);
-    element_sub(R->x, R->x, e3);
-
-    //e2 = 8y^4
-    element_square(e2, e2);
-    element_mul_si(e2, e2, 8);
-
-    //y_out = e0(e1 - x_out) - e2
-    element_sub(e1, e1, R->x);
-    element_mul(e0, e0, e1);
-    element_sub(R->y, e0, e2);
-
-    R->inf_flag = 0;
-
-    element_clear(e0);
-    element_clear(e1);
-    element_clear(e2);
-    element_clear(e3);
-}
-
-void point_proj_add(point_t R, element_t Rz, point_t P, element_t Pz,
-	point_t Q, element_t Qz)
-{
-    element_t e0, e1, e2, e3, e4, e5;
-    element_init(e0, Rz->field);
-    element_init(e1, Rz->field);
-    element_init(e2, Rz->field);
-    element_init(e3, Rz->field);
-    element_init(e4, Rz->field);
-    element_init(e5, Rz->field);
-
-    element_ptr Px = P->x;
-    element_ptr Py = P->y;
-    element_ptr Qx = Q->x;
-    element_ptr Qy = Q->y;
-
-    //e0 = Px Qz^2
-    element_square(e3, Qz);
-    element_mul(e0, Px, e3);
-
-    //e1 = Qx Pz^2
-    element_square(e4, Pz);
-    element_mul(e1, Qx, e4);
-
-    //e2 = e0 - e1
-    element_sub(e2, e0, e1);
-
-    //e3 = Py Qz^3
-    element_mul(e3, e3, Qz);
-    element_mul(e3, e3, Py);
-
-    //e4 = Qy Pz^3
-    element_mul(e4, e4, Pz);
-    element_mul(e4, e4, Qy);
-
-    //e5 = e3 - e4
-    element_sub(e5, e3, e4);
-
-    //TODO: finish this?
-    element_clear(e0);
-    element_clear(e1);
-    element_clear(e2);
-    element_clear(e3);
-    element_clear(e4);
-    element_clear(e5);
-}
-
-//convert P from weighted projective (Jacobian) to affine
-//i.e. (X, Y, Z) --> (X/Z^2, Y/Z^3)
-//also sets z to 1
-static void point_to_affine(point_ptr P, element_t z)
-{
-    element_t e0;
-
-    element_init(e0, z->field);
-    element_invert(z, z);
-    element_square(e0, z);
-    element_mul(P->x, P->x, e0);
-    element_mul(e0, e0, z);
-    element_mul(P->y, P->y, e0);
-    element_set1(z);
-
-    element_clear(e0);
-}
-
 static void a_pairing_proj(element_ptr out, element_ptr in1, element_ptr in2,
 	pairing_t pairing)
 //in1, in2 are from E(F_q), out from F_q^2
 {
     a_pairing_data_ptr p = pairing->data;
     point_t V, V1;
-    element_t z;
+    element_t z, z2;
     element_t f, f0, f1;
     element_t a, b, c;
     element_t e0;
+    const element_ptr e1 = a, e2 = b, e3 = c;
     int i, n;
     element_ptr Qx = ((point_ptr) in2->data)->x;
     element_ptr Qy = ((point_ptr) in2->data)->y;
 
+    //convert V from weighted projective (Jacobian) to affine
+    //i.e. (X, Y, Z) --> (X/Z^2, Y/Z^3)
+    //also sets z to 1
+    void point_to_affine(void)
+    {
+	element_invert(z, z);
+	element_square(e0, z);
+	element_mul(V->x, V->x, e0);
+	element_mul(e0, e0, z);
+	element_mul(V->y, V->y, e0);
+	element_set1(z);
+	element_set1(z2);
+    }
+
+    void proj_double()
+    {
+	element_ptr x = V->x;
+	element_ptr y = V->y;
+	//e0 = 3x^2 + (cc->a) z^4
+	element_square(e0, x);
+	element_mul_si(e0, e0, 3);
+	element_square(e1, z2);
+	element_add(e0, e0, e1);
+
+	//z_out = 2 y z
+	element_mul(z, y, z);
+	element_mul_si(z, z, 2);
+	element_square(z2, z);
+
+	//e1 = 4 x y^2
+	element_square(e2, y);
+	element_mul(e1, x, e2);
+	element_mul_si(e1, e1, 4);
+
+	//x_out = e0^2 - 2 e1
+	element_mul_si(e3, e1, 2);
+	element_square(x, e0);
+	element_sub(x, x, e3);
+
+	//e2 = 8y^4
+	element_square(e2, e2);
+	element_mul_si(e2, e2, 8);
+
+	//y_out = e0(e1 - x_out) - e2
+	element_sub(e1, e1, x);
+	element_mul(e0, e0, e1);
+	element_sub(y, e0, e2);
+    }
+
     void do_tangent(void) {
 	element_ptr Vx = V->x;
 	element_ptr Vy = V->y;
@@ -251,21 +189,19 @@ static void a_pairing_proj(element_ptr out, element_ptr in1, element_ptr in2,
 	//b = 2 y z^3
 	//c = -(2 y^2 + x a)
 	//a = z^2 a
-	element_square(e0, z);
-	element_square(a, e0);
+	element_square(a, z2);
 	element_square(b, Vx);
 	element_mul_si(b, b, 3);
 	element_add(a, a, b);
 	element_neg(a, a);
 
-	element_mul(b, z, e0);
-	element_mul(b, b, Vy);
-	element_mul_si(b, b, 2);
+	element_mul_si(e0, Vy, 2);
+	element_mul(b, e0, z2);
+	element_mul(b, b, z);
 
 	element_mul(c, Vx, a);
-	element_mul(a, a, e0); //frees e0, don't need z^2 anymore
-	element_square(e0, Vy);
-	element_mul_si(e0, e0, 2);
+	element_mul(a, a, z2);
+	element_mul(e0, e0, Vy);
 	element_add(c, c, e0);
 	element_neg(c, c);
 
@@ -313,16 +249,18 @@ static void a_pairing_proj(element_ptr out, element_ptr in1, element_ptr in2,
     element_init(c, p->Fq);
     element_init(e0, p->Fq);
     element_init(z, p->Fq);
+    element_init(z2, p->Fq);
     element_set1(z);
+    element_set1(z2);
     n = p->exp1;
     for (i=0; i<n; i++) {
 	//f = f^2 g_V,V(Q)
 	//where g_V,V = tangent at V
 	element_square(f, f);
 	do_tangent();
-	point_proj_double(V, z, V, z);
+	proj_double();
     }
-    point_to_affine(V, z);
+    point_to_affine();
     if (p->sign1 < 0) {
 	point_neg(V1, V);
 	element_invert(f1, f);
@@ -334,11 +272,11 @@ static void a_pairing_proj(element_ptr out, element_ptr in1, element_ptr in2,
     for (; i<n; i++) {
 	element_square(f, f);
 	do_tangent();
-	point_proj_double(V, z, V, z);
+	proj_double();
     }
 
     element_mul(f, f, f1);
-    point_to_affine(V, z);
+    point_to_affine();
     do_line(V, V1);
 
     //Tate exponentiation
@@ -354,6 +292,7 @@ static void a_pairing_proj(element_ptr out, element_ptr in1, element_ptr in2,
     element_clear(f0);
     element_clear(f1);
     element_clear(z);
+    element_clear(z2);
     point_clear(V);
     point_clear(V1);
     element_clear(a);
@@ -499,7 +438,7 @@ void pairing_init_a_param(pairing_t pairing, a_param_t param)
     mpz_init(pairing->r);
     mpz_set(pairing->r, param->r);
     field_init_fp(pairing->Zr, pairing->r);
-    pairing->map = a_pairing;
+    pairing->map = a_pairing_proj;
     pairing->is_almost_coddh = generic_is_almost_coddh;
 
     field_init_fp(p->Fq, param->q);
