@@ -60,16 +60,16 @@ typedef struct field_s field_t[1];
 
 typedef void (*fieldmap)(element_t dst, element_t src);
 
-static inline void element_init(element_ptr e, field_ptr f)
+static inline void realelement_init(element_ptr e, field_ptr f)
 {
     e->field = f;
     f->init(e);
 }
 
-static inline void element_clear(element_ptr e)
+static inline void realelement_clear(element_ptr e)
 {
     e->field->clear(e);
-    e->field = NULL;
+    //e->field = NULL;
 }
 
 static inline void element_out_str(FILE *stream, int base, element_ptr e)
